(* type t *)
type 'a t = Empty | Node of ('a t * 'a * 'a t)
(* size : 'a t -> int *)
let rec size = function Empty       -> 0
                      | Node(l,_,r) -> size l + size r + 1
(* height : 'a t -> int *)
let rec height = function
 | Empty       -> 0
 | Node(l,_,r) -> max (height l) (height r) + 1
(* of_list : 'a list -> 'a t *)
let rec of_list = function []    -> Empty
                         | x::xs -> Node(Empty,x,of_list xs)
(* make : 'a list -> 'a t *)
let rec make = function
 | [] -> Empty
 | xs ->
  let m = Lst.length xs / 2 in
  let (ys,zs) = Lst.split_at m xs in
  Node (make ys,Lst.hd zs,make(Lst.tl zs))
(* insert : ('a -> 'a -> int) -> 'a -> 'a t -> 'a t *)
let rec insert c v = function
 | Empty       -> Node(Empty,v,Empty)
 | Node(l,w,r) -> if c v w < 1 then Node(insert c v l,w,r)
                               else Node(l,w,insert c v r)
(* search_tree ('a -> 'a -> int) -> 'a list -> 'a t *)
let search_tree c = Lst.foldl (fun t v -> insert c v t) Empty
(* flatten : 'a t -> 'a list *)
let rec flatten = function
 | Empty       -> []
 | Node(l,v,r) -> (flatten l)@(v::flatten r)
(* *)
(* pretty printing *)
let rec phantom d = if d = 0 then Empty else Node(phantom (d-1),None,phantom (d-1));;

let rec balance d = function
 | Empty -> phantom d
 | Node(l,v,r) -> Node(balance (d-1) l, Some v, balance (d-1) r)

let level = function 
 | Empty -> ([None],[])
 | Node(l,v,r) -> ([v],[l;r])

let rec levels p fmt i = function
 | [] -> ()
 | xs -> 
  let ws = Lst.map level xs in
  let ss = Lst.map p (Lst.concat (Lst.map fst ws)) in
  let line = Strng.blanks (i/2-1)@Lst.join (Strng.blanks (i-1)) ss in
  Format.fprintf fmt "%s\n" (Strng.to_string line);
  levels p fmt (i/2) (Lst.concat(Lst.map snd ws))

let printer p fmt t = 
 Format.fprintf fmt "\n";
 let t = balance (height t) t in 
 levels p fmt (Int.pow2 (height t)) [t]

let intnode2strng = function
  | None -> [' ']
  | Some v -> Strng.of_int v

let toplevel_printer fmt t = (* nice output for int trees *)
 printer intnode2strng fmt t
