(* W01 *)
(* W03 *)
(* type t *)
type t = char list
(* of_string : string -> char list *)
let of_string s =
 let rec of_string i acc =
  if i < 0 then acc else of_string (i-1) (s.[i]::acc)
 in
 of_string (String.length s - 1) []
(* to_string : char list -> string *)
let to_string xs =
 let buffer = Buffer.create 128 in
 List.iter (Buffer.add_char buffer) xs;
 Buffer.contents buffer
(* of_int : int -> char list *)
let of_int i = of_string(string_of_int i)
(* print : char list -> unit *)
let print s = Printf.printf "%s" (to_string s)
(* toplevel_printer : Format.formatter -> char list -> unit *)
let toplevel_printer fmt s =
 Format.fprintf fmt "\"%s\"" (String.escaped(to_string s))
(* blanks : int -> t *)
let blanks i = Lst.replicate i ' '
(* E03 *)

(* left_justify : int -> t -> t *)
let left_justify i s = s @ blanks (i - Lst.length s)

(* right_justify : int -> t -> t *)
let right_justify i s = blanks (i - Lst.length s) @ s

(* center : int -> t -> t *)
let center i s = let m = i - Lst.length s in
 blanks (m/2) @ s @ blanks (m/2 mod 2)

(* is_white_space : char -> bool *)
let is_white_space x = Lst.mem x [' ';'\n';'\t'];;

(* words : t -> t list *)
let rec words = function
 | [] -> []
 | cs -> let (w,cs) = Lst.span (fun c -> not (is_white_space c)) cs in
  let (_,cs) = Lst.span is_white_space cs in
  w::words cs

(* break : int -> t -> t list *)
let rec break i cs = 
 if Lst.length cs <= i then [cs]
 else Lst.take i cs::break i (Lst.drop i cs)

(* lines : int -> t -> t list *)
let lines i cs =
 let rec p i acc = function
   [] -> [acc]
   | w::ws ->  
    let line = acc @ [' '] @ w in
    if Lst.length line > i then [acc]@p i w ws else p i line ws
 in
 match Lst.concat (Lst.map (break i) (words cs)) with
  | [] -> []
  | w::ws -> p i w ws

(* *)
